/*
 * Copyright (C) 2012 CERN (www.cern.ch)
 * Author: Aurelio Colosimo
 * Based on ptp-noposix project (see AUTHORS for details)
 *
 * Released to the public domain
 */

#include <ppsi/ppsi.h>
#include "wr-api.h"

/*
 * We enter here from WRS_CALIBRATION.  If master we wait for
 * a CALIBRATE message, if slave we wait for LINK_ON.
 */
int wr_calibrated(struct pp_instance *ppi, unsigned char *pkt, int plen)
{
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
                /* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_calibrated_1++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
	struct wr_dsport *wrp = WR_DSPOR(ppi);
	MsgSignaling wrsig_msg;

	if (ppi->is_new_state)
		__pp_timeout_set(ppi, PP_TO_EXT_0, wrp->wrStateTimeout);

	if (pp_timeout(ppi, PP_TO_EXT_0)) {
		/*
		 * FIXME: We should implement a retry by re-sending
		 * the "calibrated" message, moving it here from the
		 * previous state (sub-state 8 of "state-wr-calibration"
		 */
		wr_handshake_fail(ppi);
		return 0; /* non-wr */
	}
#ifdef BROADCAST_NODE
	// In broadcast mode we don't wait for WR_MODE_ON message.
	ppi->next_state = WRS_WR_LINK_ON;
#else
	if (ppi->received_ptp_header.messageType == PPM_SIGNALING) {
		msg_unpack_wrsig(ppi, pkt, &wrsig_msg,
			 &(wrp->msgTmpWrMessageID));

		if ((wrp->msgTmpWrMessageID == CALIBRATE) &&
			(wrp->wrMode == WR_MASTER))
			ppi->next_state = WRS_RESP_CALIB_REQ;
		else if ((wrp->msgTmpWrMessageID == WR_MODE_ON) &&
			(wrp->wrMode == WR_SLAVE))
		{
#ifdef BROADCAST_BASE
			if (memcmp(ppi->received_ptp_header.targetIdentity.clockIdentity.id,
				   DSPOR(ppi)->portIdentity.clockIdentity.id,
				   PP_CLOCK_IDENTITY_LENGTH)==0)
			{
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
                		/* ONLY FOR TESTS */
                		WR_DSPOR(ppi)->ctr_wr_calibrated_2++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
				ppi->next_state = WRS_WR_LINK_ON;
			}
#else
			ppi->next_state = WRS_WR_LINK_ON;
#endif // BROADCAST_BASE
		}
	}
#endif // BROADCAST_NODE

	ppi->next_delay = wrp->wrStateTimeout;
	return 0;
}
