/*
 * Copyright (C) 2012 CERN (www.cern.ch)
 * Author: Aurelio Colosimo
 * Based on ptp-noposix project (see AUTHORS for details)
 *
 * Released to the public domain
 */

#include <ppsi/ppsi.h>
#include "wr-api.h"
#include "../proto-standard/common-fun.h"

/*
 * WRS_PRESENT is the entry point for a WR slave
 *
 * Here we send SLAVE_PRESENT and wait for LOCK. If timeout,
 * resent SLAVE_PRESENT from WR_STATE_RETRY times
 */
int wr_present(struct pp_instance *ppi, unsigned char *pkt, int plen)
{
	int e = 0, sendmsg = 0;
	struct wr_dsport *wrp = WR_DSPOR(ppi);

	MsgSignaling wrsig_msg;

	if (ppi->is_new_state) {
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
		/* ONLY FOR TESTS */
		WR_DSPOR(ppi)->ctr_wr_present_0_1++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
		wrp->wrStateRetry = WR_STATE_RETRY;
		sendmsg = 1;
	} else if (pp_timeout(ppi, PP_TO_EXT_0)) {
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
		/* ONLY FOR TESTS */
		WR_DSPOR(ppi)->ctr_wr_present_0_2++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
		if (wr_handshake_retry(ppi))
			sendmsg = 1;
		else
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
		/* ONLY FOR TESTS */
		WR_DSPOR(ppi)->ctr_wr_present_0_3++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
		return 0; /* non-wr already */
	}

#ifdef BROADCAST_NODE
	// In broadcast mode we don't send SLAVE_PRESENT message & don't wait for the LOCK message.
	ppi->next_state = WRS_S_LOCK;
#else
	if (sendmsg) {
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
		/* ONLY FOR TESTS */
		WR_DSPOR(ppi)->ctr_wr_present_0_4++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
		__pp_timeout_set(ppi, PP_TO_EXT_0, WR_WRS_PRESENT_TIMEOUT_MS);
		e = msg_issue_wrsig(ppi, SLAVE_PRESENT);
	}

	if (ppi->received_ptp_header.messageType == PPM_SIGNALING) {
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
		/* ONLY FOR TESTS */
		WR_DSPOR(ppi)->ctr_wr_present_1++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
		msg_unpack_wrsig(ppi, pkt, &wrsig_msg,
			 &(wrp->msgTmpWrMessageID));
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
		/* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_present_2++;
#endif // BROADCAST_BASE && BC_EXTRA_MON

		if (wrp->msgTmpWrMessageID == LOCK)
		{
#ifdef BROADCAST_BASE
#ifdef BC_EXTRA_MON
                	/* ONLY FOR TESTS */
                	WR_DSPOR(ppi)->ctr_wr_present_3++;
#endif // BC_EXTRA_MON
			if (memcmp(ppi->received_ptp_header.targetIdentity.clockIdentity.id,
					DSPOR(ppi)->portIdentity.clockIdentity.id,
					PP_CLOCK_IDENTITY_LENGTH)==0)
			{
#ifdef BC_EXTRA_MON
				/* ONLY FOR TESTS */
                		WR_DSPOR(ppi)->ctr_wr_present_4++;
#endif // BC_EXTRA_MON
				ppi->next_state = WRS_S_LOCK;
			}
#else
			ppi->next_state = WRS_S_LOCK;
#endif // BROADCAST_BASE
		}
	}
#endif // BROADCAST_NODE

	if (e == 0)
	{
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
		/* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_present_00_1++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
		st_com_execute_slave(ppi);
	}
	else {
		/* nothing, just stay here again */
	}
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
		/* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_present_00_2++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
	ppi->next_delay = WR_DSPOR(ppi)->wrStateTimeout;

	return e;
}
