/*
 * Copyright (C) 2012 CERN (www.cern.ch)
 * Author: Aurelio Colosimo
 * Based on ptp-noposix project (see AUTHORS for details)
 *
 * Released to the public domain
 */

#include <ppsi/ppsi.h>
#include "wr-api.h"

int wr_resp_calib_req(struct pp_instance *ppi, unsigned char *pkt, int plen)
{
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
                /* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_resp_calib_req_1++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
	struct wr_dsport *wrp = WR_DSPOR(ppi);
	MsgSignaling wrsig_msg;
	int e = 0, enable = 0;
	int send_pattern = (wrp->otherNodeCalSendPattern != 0);

	if (ppi->is_new_state) {
		wrp->wrStateRetry = WR_STATE_RETRY;
		enable = 1;
	} else if (pp_timeout(ppi, PP_TO_EXT_0)) {
		if (send_pattern)
			wrp->ops->calib_pattern_disable(ppi);
		if (wr_handshake_retry(ppi))
			enable = 1;
		else
			return 0; /* non-wr already */
	}

	if (enable) { /* first or retry */
		if (send_pattern)
			wrp->ops->calib_pattern_enable(ppi, 0, 0, 0);
		__pp_timeout_set(ppi, PP_TO_EXT_0,
			       wrp->otherNodeCalPeriod / 1000);
	}

#ifdef BROADCAST_NODE
	// In broadcast mode we don't wait for CALIBRATED message.
	ppi->next_state = WRS_CALIBRATION;
#else
	if (ppi->received_ptp_header.messageType == PPM_SIGNALING) {

		msg_unpack_wrsig(ppi, pkt, &wrsig_msg,
			 &(wrp->msgTmpWrMessageID));

		if (wrp->msgTmpWrMessageID == CALIBRATED) {
#ifdef BROADCAST_BASE
			if (memcmp(ppi->received_ptp_header.targetIdentity.clockIdentity.id,
			    DSPOR(ppi)->portIdentity.clockIdentity.id,
			    PP_CLOCK_IDENTITY_LENGTH)==0)
			{
				if (send_pattern)
					wrp->ops->calib_pattern_disable(ppi);
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
                		/* ONLY FOR TESTS */
                		WR_DSPOR(ppi)->ctr_wr_resp_calib_req_2++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
				ppi->next_state = WRS_CALIBRATION;
			}
#else
			if (send_pattern)
				wrp->ops->calib_pattern_disable(ppi);
			if (wrp->wrMode == WR_MASTER)
				ppi->next_state = WRS_WR_LINK_ON;
			else
				ppi->next_state = WRS_CALIBRATION;
#endif // BROADCAST_BASE
		}
	}
#endif // BROADCAST_NODE

	ppi->next_delay = wrp->wrStateTimeout;
	return e;
}
