/*
 * Copyright (C) 2012 CERN (www.cern.ch)
 * Author: Aurelio Colosimo
 * Based on ptp-noposix project (see AUTHORS for details)
 *
 * Released to the public domain
 */

#include <ppsi/ppsi.h>
#include "wr-api.h"

int wr_s_lock(struct pp_instance *ppi, unsigned char *pkt, int plen)
{
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
                /* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_s_lock_1++;
#endif // BROADCAST_BASE && BC_EXTRA_MON

	struct wr_dsport *wrp = WR_DSPOR(ppi);
	int enable = 0;
	int poll_ret;

	if (ppi->is_new_state) {
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
                /* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_s_lock_2++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
                wrp->wrStateRetry = WR_STATE_RETRY;
		enable = 1;
	} else if (pp_timeout(ppi, PP_TO_EXT_0)) {
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
                /* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_s_lock_3++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
		wrp->ops->locking_disable(ppi);
		if (wr_handshake_retry(ppi))
		{
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
                /* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_s_lock_4++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
			enable = 1;
		}
		else
			return 0; /* non-wr already */
	}

	if (enable) {
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
                /* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_s_lock_5++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
		wrp->ops->locking_enable(ppi);
		__pp_timeout_set(ppi, PP_TO_EXT_0, WR_S_LOCK_TIMEOUT_MS);
	}

	ppi->next_delay = wrp->wrStateTimeout;

	poll_ret = wrp->ops->locking_poll(ppi, 0);
	if (poll_ret == WR_SPLL_READY) {
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
                /* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_s_lock_6++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
		ppi->next_state = WRS_LOCKED;
		wrp->ops->locking_disable(ppi);
	} else if (poll_ret == WR_SPLL_CALIB_NOT_READY) {
#if defined(BROADCAST_BASE) && defined(BC_EXTRA_MON)
                /* ONLY FOR TESTS */
                WR_DSPOR(ppi)->ctr_wr_s_lock_7++;
#endif // BROADCAST_BASE && BC_EXTRA_MON
		/* rxts_calibration not ready, enter the same state without
		 * a delay */
		ppi->next_delay = 0;
	}

	return 0;
}
